import sys
import json
from sqlalchemy import create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String
from sqlalchemy import Float, Date, DateTime, ForeignKey
from sqlalchemy.orm import sessionmaker
import datetime
from pyexcel_io._compact import OrderedDict
from pyexcel_io.database.common import (
    SQLTableExporter,
    SQLTableExportAdapter,
    SQLTableImporter,
    SQLTableImportAdapter,
)
from pyexcel_io.database.exporters.sqlalchemy import (
    SQLTableReader,
    SQLBookReader,
)
from pyexcel_io.database.importers.sqlalchemy import (
    PyexcelSQLSkipRowException,
    SQLTableWriter,
    SQLBookWriter,
)
from pyexcel_io.database.querysets import QuerysetsReader
from sqlalchemy.orm import relationship, backref
from nose.tools import raises, eq_
import platform


PY3 = sys.version_info[0] == 3
PY36 = PY3 and sys.version_info[1] == 6


engine = None
if platform.python_implementation() == "PyPy":
    engine = create_engine("sqlite:///tmp.db")
else:
    engine = create_engine("sqlite://")

Base = declarative_base()


class Pyexcel(Base):
    __tablename__ = "pyexcel"
    id = Column(Integer, primary_key=True)
    name = Column(String, unique=True)
    weight = Column(Float)
    birth = Column(Date)


class Post(Base):
    __tablename__ = "post"
    id = Column(Integer, primary_key=True)
    title = Column(String(80))
    body = Column(String(100))
    pub_date = Column(DateTime)

    category_id = Column(Integer, ForeignKey("category.id"))
    category = relationship(
        "Category", backref=backref("posts", lazy="dynamic")
    )

    def __init__(self, title, body, category, pub_date=None):
        self.title = title
        self.body = body
        if pub_date is None:
            pub_date = datetime.utcnow()
        self.pub_date = pub_date
        self.category = category

    def __repr__(self):
        return "<Post %r>" % self.title


class Category(Base):
    __tablename__ = "category"
    id = Column(Integer, primary_key=True)
    name = Column(String(50))

    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return "<Category %r>" % self.name

    def __str__(self):
        return self.__repr__()


Session = sessionmaker(bind=engine)


class TestSingleRead:
    def setUp(self):
        Base.metadata.drop_all(engine)
        Base.metadata.create_all(engine)
        p1 = Pyexcel(
            id=0, name="Adam", weight=11.25, birth=datetime.date(2014, 11, 11)
        )
        self.session = Session()
        self.session.add(p1)
        p1 = Pyexcel(
            id=1, name="Smith", weight=12.25, birth=datetime.date(2014, 11, 12)
        )
        self.session.add(p1)
        self.session.commit()
        self.session.close()

    def test_sql(self):
        mysession = Session()
        sheet = SQLTableReader(mysession, Pyexcel)
        data = sheet.to_array()
        content = [
            ["birth", "id", "name", "weight"],
            ["2014-11-11", 0, "Adam", 11.25],
            ["2014-11-12", 1, "Smith", 12.25],
        ]
        # 'pyexcel' here is the table name
        assert list(data) == content
        mysession.close()

    def test_sql_formating(self):
        mysession = Session()

        def custom_renderer(row):
            return [str(element) for element in row]

        # the key for this test case
        sheet = SQLTableReader(
            mysession, Pyexcel, row_renderer=custom_renderer
        )
        data = sheet.to_array()
        content = [
            ["birth", "id", "name", "weight"],
            ["2014-11-11", "0", "Adam", "11.25"],
            ["2014-11-12", "1", "Smith", "12.25"],
        ]
        eq_(list(data), content)
        mysession.close()

    def test_sql_filter(self):
        mysession = Session()
        sheet = SQLTableReader(mysession, Pyexcel, start_row=1)
        data = sheet.to_array()
        content = [
            ["2014-11-11", 0, "Adam", 11.25],
            ["2014-11-12", 1, "Smith", 12.25],
        ]
        # 'pyexcel'' here is the table name
        assert list(data) == content
        mysession.close()

    def test_sql_filter_1(self):
        mysession = Session()
        sheet = SQLTableReader(mysession, Pyexcel, start_row=1, row_limit=1)
        data = sheet.to_array()
        content = [["2014-11-11", 0, "Adam", 11.25]]
        # 'pyexcel'' here is the table name
        assert list(data) == content
        mysession.close()

    def test_sql_filter_2(self):
        mysession = Session()
        sheet = SQLTableReader(mysession, Pyexcel, start_column=1)
        data = sheet.to_array()
        content = [
            ["id", "name", "weight"],
            [0, "Adam", 11.25],
            [1, "Smith", 12.25],
        ]
        # 'pyexcel'' here is the table name
        assert list(data) == content
        mysession.close()

    def test_sql_filter_3(self):
        mysession = Session()
        sheet = SQLTableReader(
            mysession, Pyexcel, start_column=1, column_limit=1
        )
        data = sheet.to_array()
        content = [["id"], [0], [1]]
        # 'pyexcel'' here is the table name
        assert list(data) == content
        mysession.close()


class TestSingleWrite:
    def setUp(self):
        Base.metadata.drop_all(engine)
        Base.metadata.create_all(engine)
        self.data = [
            ["birth", "id", "name", "weight"],
            [datetime.date(2014, 11, 11), 0, "Adam", 11.25],
            [datetime.date(2014, 11, 12), 1, "Smith", 12.25],
        ]
        self.results = [
            ["birth", "id", "name", "weight"],
            ["2014-11-11", 0, "Adam", 11.25],
            ["2014-11-12", 1, "Smith", 12.25],
        ]

    def test_one_table(self):
        mysession = Session()
        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = self.data[0]
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(self.data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        reader = QuerysetsReader(query_sets, self.data[0])
        results = reader.to_array()
        assert list(results) == self.results
        mysession.close()

    def test_update_existing_row(self):
        mysession = Session()
        # write existing data
        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = self.data[0]
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(self.data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        results = QuerysetsReader(query_sets, self.data[0]).to_array()
        assert list(results) == self.results
        # update data using custom initializer
        update_data = [
            ["birth", "id", "name", "weight"],
            [datetime.date(2014, 11, 11), 0, "Adam_E", 12.25],
            [datetime.date(2014, 11, 12), 1, "Smith_E", 11.25],
        ]
        updated_results = [
            ["birth", "id", "name", "weight"],
            ["2014-11-11", 0, "Adam_E", 12.25],
            ["2014-11-12", 1, "Smith_E", 11.25],
        ]

        def row_updater(row):
            an_instance = mysession.query(Pyexcel).get(row["id"])
            if an_instance is None:
                an_instance = Pyexcel()
            for name in row.keys():
                setattr(an_instance, name, row[name])
            return an_instance

        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = update_data[0]
        adapter.row_initializer = row_updater
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(update_data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        results = QuerysetsReader(query_sets, self.data[0]).to_array()
        assert list(results) == updated_results
        mysession.close()

    def test_skipping_rows_if_data_exist(self):
        mysession = Session()
        # write existing data
        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = self.data[0]
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(self.data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        results = QuerysetsReader(query_sets, self.data[0]).to_array()
        assert list(results) == self.results
        # update data using custom initializer
        update_data = [
            ["birth", "id", "name", "weight"],
            [datetime.date(2014, 11, 11), 0, "Adam_E", 12.25],
            [datetime.date(2014, 11, 12), 1, "Smith_E", 11.25],
        ]

        def row_updater(row):
            an_instance = mysession.query(Pyexcel).get(row["id"])
            if an_instance is not None:
                raise PyexcelSQLSkipRowException()

        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = update_data[0]
        adapter.row_initializer = row_updater
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(update_data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        results = QuerysetsReader(query_sets, self.data[0]).to_array()
        assert list(results) == self.results
        mysession.close()

    def test_one_table_with_empty_rows(self):
        mysession = Session()
        data = [
            ["birth", "id", "name", "weight"],
            ["", "", ""],
            [datetime.date(2014, 11, 11), 0, "Adam", 11.25],
            [datetime.date(2014, 11, 12), 1, "Smith", 12.25],
        ]
        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = data[0]
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        results = QuerysetsReader(query_sets, data[0]).to_array()
        assert list(results) == self.results
        mysession.close()

    def test_one_table_with_empty_string_in_unique_field(self):
        mysession = Session()
        data = [
            ["birth", "id", "name", "weight"],
            [datetime.date(2014, 11, 11), 0, "", 11.25],
            [datetime.date(2014, 11, 12), 1, "", 12.25],
        ]
        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = data[0]
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        results = QuerysetsReader(query_sets, data[0]).to_array()
        assert list(results) == [
            ["birth", "id", "name", "weight"],
            ["2014-11-11", 0, None, 11.25],
            ["2014-11-12", 1, None, 12.25],
        ]
        mysession.close()

    def test_one_table_using_mapdict_as_array(self):
        mysession = Session()
        self.data = [
            ["Birth Date", "Id", "Name", "Weight"],
            [datetime.date(2014, 11, 11), 0, "Adam", 11.25],
            [datetime.date(2014, 11, 12), 1, "Smith", 12.25],
        ]
        mapdict = ["birth", "id", "name", "weight"]

        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = self.data[0]
        adapter.column_name_mapping_dict = mapdict
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(self.data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        results = QuerysetsReader(query_sets, mapdict).to_array()
        assert list(results) == self.results
        mysession.close()

    def test_one_table_using_mapdict_as_dict(self):
        mysession = Session()
        self.data = [
            ["Birth Date", "Id", "Name", "Weight"],
            [datetime.date(2014, 11, 11), 0, "Adam", 11.25],
            [datetime.date(2014, 11, 12), 1, "Smith", 12.25],
        ]
        mapdict = {
            "Birth Date": "birth",
            "Id": "id",
            "Name": "name",
            "Weight": "weight",
        }

        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = self.data[0]
        adapter.column_name_mapping_dict = mapdict
        writer = SQLTableWriter(importer, adapter)
        writer.write_array(self.data[1:])
        writer.close()
        query_sets = mysession.query(Pyexcel).all()
        results = QuerysetsReader(
            query_sets, ["birth", "id", "name", "weight"]
        ).to_array()
        assert list(results) == self.results
        mysession.close()


class TestMultipleRead:
    def setUp(self):
        Base.metadata.drop_all(engine)
        Base.metadata.create_all(engine)
        self.session = Session()
        data = {
            "Category": [["id", "name"], [1, "News"], [2, "Sports"]],
            "Post": [
                ["id", "title", "body", "pub_date", "category"],
                [
                    1,
                    "Title A",
                    "formal",
                    datetime.datetime(2015, 1, 20, 23, 28, 29),
                    "News",
                ],
                [
                    2,
                    "Title B",
                    "informal",
                    datetime.datetime(2015, 1, 20, 23, 28, 30),
                    "Sports",
                ],
            ],
        }

        def category_init_func(row):
            c = Category(row["name"])
            c.id = row["id"]
            return c

        def post_init_func(row):
            c = (
                self.session.query(Category)
                .filter_by(name=row["category"])
                .first()
            )
            p = Post(row["title"], row["body"], c, row["pub_date"])
            return p

        importer = SQLTableImporter(self.session)
        category_adapter = SQLTableImportAdapter(Category)
        category_adapter.column_names = data["Category"][0]
        category_adapter.row_initializer = category_init_func
        importer.append(category_adapter)
        post_adapter = SQLTableImportAdapter(Post)
        post_adapter.column_names = data["Post"][0]
        post_adapter.row_initializer = post_init_func
        importer.append(post_adapter)
        writer = SQLBookWriter()
        writer.open_content(importer)
        to_store = OrderedDict()
        to_store.update({category_adapter.get_name(): data["Category"][1:]})
        to_store.update({post_adapter.get_name(): data["Post"][1:]})
        writer.write(to_store)
        writer.close()

    def test_read(self):
        exporter = SQLTableExporter(self.session)
        category_adapter = SQLTableExportAdapter(Category)
        exporter.append(category_adapter)
        post_adapter = SQLTableExportAdapter(Post)
        exporter.append(post_adapter)
        book = SQLBookReader()
        book.open_content(exporter)
        data = book.read_all()
        for key in data.keys():
            data[key] = list(data[key])
        assert json.dumps(data) == (
            '{"category": [["id", "name"], [1, "News"], [2, "Sports"]], '
            + '"post": [["body", "category_id", "id", "pub_date", "title"], '
            + '["formal", 1, 1, "2015-01-20T23:28:29", "Title A"], '
            + '["informal", 2, 2, "2015-01-20T23:28:30", "Title B"]]}'
        )

    def test_foreign_key(self):
        all_posts = self.session.query(Post).all()
        column_names = ["category__name", "title"]
        data = list(QuerysetsReader(all_posts, column_names).to_array())
        eq_(
            json.dumps(data),
            '[["category__name", "title"], ["News", "Title A"],'
            + ' ["Sports", "Title B"]]',
        )

    def tearDown(self):
        self.session.close()


class TestZeroRead:
    def setUp(self):
        Base.metadata.drop_all(engine)
        Base.metadata.create_all(engine)

    def test_sql(self):
        mysession = Session()
        sheet = SQLTableReader(mysession, Pyexcel)
        data = sheet.to_array()
        content = [[], []]
        # 'pyexcel' here is the table name
        assert list(data) == content
        mysession.close()


class TestNoAutoCommit:
    def setUp(self):
        Base.metadata.drop_all(engine)
        Base.metadata.create_all(engine)
        self.data = [
            ["birth", "id", "name", "weight"],
            [datetime.date(2014, 11, 11), 0, "Adam", 11.25],
            [datetime.date(2014, 11, 12), 1, "Smith", 12.25],
        ]
        self.results = [
            ["birth", "id", "name", "weight"],
            ["2014-11-11", 0, "Adam", 11.25],
            ["2014-11-12", 1, "Smith", 12.25],
        ]

    def test_one_table(self):
        if PY36:
            # skip the test
            # beause python 3.6 sqlite give segmentation fault
            return
        mysession = Session()
        importer = SQLTableImporter(mysession)
        adapter = SQLTableImportAdapter(Pyexcel)
        adapter.column_names = self.data[0]
        writer = SQLTableWriter(importer, adapter, auto_commit=False)
        writer.write_array(self.data[1:])
        writer.close()
        mysession.close()
        mysession2 = Session()
        query_sets = mysession2.query(Pyexcel).all()
        eq_(len(query_sets), 0)
        mysession2.close()


@raises(TypeError)
def test_not_implemented_method():
    reader = SQLBookReader()
    reader.open("afile")


@raises(TypeError)
def test_not_implemented_method_2():
    reader = SQLBookReader()
    reader.open_stream("afile")


def test_sql_table_import_adapter():
    adapter = SQLTableImportAdapter(Pyexcel)
    adapter.column_names = ["a"]
    adapter.row_initializer = "abc"
    eq_(adapter.row_initializer, "abc")


@raises(Exception)
def test_unknown_sheet(self):
    importer = SQLTableImporter(None)
    category_adapter = SQLTableImportAdapter(Category)
    category_adapter.column_names = [""]
    importer.append(category_adapter)
    writer = SQLBookWriter()
    writer.open_content(importer)
    to_store = OrderedDict()
    to_store.update({"you do not see me": [[]]})
    writer.write(to_store)



"""
    pyexcel_io.fileformat.csvz
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    The lower level csvz file format handler.

    :copyright: (c) 2014-2017 by Onni Software Ltd.
    :license: New BSD License, see LICENSE for more details
"""
import zipfile

from pyexcel_io._compact import StringIO, PY2
from pyexcel_io.book import BookWriter
from pyexcel_io.constants import DEFAULT_SHEET_NAME, FILE_FORMAT_CSVZ

from .csvw import CSVSheetWriter, UnicodeWriter


class CSVZipSheetWriter(CSVSheetWriter):
    """ handle the zipfile interface """

    def __init__(self, zipfile, sheetname, file_extension, **keywords):
        self.file_extension = file_extension
        keywords["single_sheet_in_book"] = False
        CSVSheetWriter.__init__(self, zipfile, sheetname, **keywords)

    def set_sheet_name(self, name):
        self.content = StringIO()
        if PY2:
            self.writer = UnicodeWriter(
                self.content, encoding=self._encoding, **self._keywords
            )
        else:
            import csv

            self.writer = csv.writer(self.content, **self._keywords)

    def close(self):
        file_name = "%s.%s" % (self._native_sheet, self.file_extension)
        self.content.seek(0)
        self._native_book.writestr(file_name, self.content.read())
        self.content.close()


class CSVZipBookWriter(BookWriter):
    """
    csvz writer

    It is better to store csv files as a csvz as it saves your disk space.
    Pyexcel-io had the facility to unzip it for you or you could use
    any other unzip software.
    """

    def __init__(self):
        BookWriter.__init__(self)
        self._file_type = FILE_FORMAT_CSVZ
        self.zipfile = None

    def open(self, file_name, **keywords):
        BookWriter.open(self, file_name, **keywords)
        self.zipfile = zipfile.ZipFile(file_name, "w", zipfile.ZIP_DEFLATED)

    def create_sheet(self, name):
        given_name = name
        if given_name is None:
            given_name = DEFAULT_SHEET_NAME
        writer = CSVZipSheetWriter(
            self.zipfile, given_name, self._file_type[:3], **self._keywords
        )
        return writer

    def close(self):
        self.zipfile.close()



from nose.tools import eq_
from pyexcel_io.sheet import SheetWriter, SheetReader
import pyexcel_io.constants as constants


class MyWriter(SheetWriter):
    def set_size(self, size):
        self.native_book = size


class MyReader(SheetReader):
    def number_of_rows(self):
        return len(self._native_sheet)

    def number_of_columns(self):
        return len(self._native_sheet[0])

    def cell_value(self, row, column):
        return self._native_sheet[row][column]


def test_write_empty_array():
    test_string = "somebook"
    writer = MyWriter(test_string, "somesheet", "somename")
    writer.write_array([])
    eq_(test_string, writer._native_book)


def take_second_column(current_index, start, limit=-1):
    decision = constants.SKIP_DATA
    if current_index == 1:
        decision = constants.TAKE_DATA
    elif current_index > 1:
        decision = constants.STOP_ITERATION
    return decision


def test_custom_skip_row_func():
    take_second_row = take_second_column
    array = [[1, 2, 3], [4, 5, 6]]
    reader = MyReader(array, skip_row_func=take_second_row)
    actual = list(reader.to_array())
    expected = [[4, 5, 6]]
    eq_(expected, actual)
    reader.close()


def test_custom_skip_column_func():
    array = [[1, 2, 3], [4, 5, 6]]
    reader = MyReader(array, skip_column_func=take_second_column)
    actual = list(reader.to_array())
    expected = [[2], [5]]
    eq_(expected, actual)
    reader.close()



#!/usr/bin/env python3

# Template by pypi-mobans
import os
import sys
import codecs
import locale
import platform
from shutil import rmtree

from setuptools import Command, setup, find_packages

PY2 = sys.version_info[0] == 2
PY26 = PY2 and sys.version_info[1] < 7
PY33 = sys.version_info < (3, 4)

# Work around mbcs bug in distutils.
# http://bugs.python.org/issue10945
# This work around is only if a project supports Python < 3.4

# Work around for locale not being set
try:
    lc = locale.getlocale()
    pf = platform.system()
    if pf != 'Windows' and lc == (None, None):
        locale.setlocale(locale.LC_ALL, 'C.UTF-8')
except (ValueError, UnicodeError, locale.Error):
    locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')

NAME = 'pyexcel-io'
AUTHOR = 'C.W.'
VERSION = '0.5.13'
EMAIL = 'wangc_2011@hotmail.com'
LICENSE = 'New BSD'
DESCRIPTION = (
    'A python library to read and write structured data in csv, zipped csv' +
    'format and to/from databases'
)
URL = 'https://github.com/pyexcel/pyexcel-io'
DOWNLOAD_URL = '%s/archive/0.5.13.tar.gz' % URL
FILES = ['README.rst', 'CHANGELOG.rst']
KEYWORDS = [
    'python',
    'API',
    'tsv',
    'tsvz',
    'csv',
    'csvz',
    'django',
    'sqlalchemy',
]

CLASSIFIERS = [
    'Topic :: Software Development :: Libraries',
    'Programming Language :: Python',
    'Intended Audience :: Developers',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'License :: OSI Approved :: BSD License',
    'Programming Language :: Python :: Implementation :: PyPy'
]

INSTALL_REQUIRES = [
    'lml>=0.0.4',
]
SETUP_COMMANDS = {}

if PY26:
    INSTALL_REQUIRES.append('ordereddict')

PACKAGES = find_packages(exclude=['ez_setup', 'examples', 'tests'])
EXTRAS_REQUIRE = {
    'xls': ['pyexcel-xls>=0.5.0'],
    'xlsx': ['pyexcel-xlsx>=0.5.0'],
    'ods': ['pyexcel-ods3>=0.5.0'],
}
# You do not need to read beyond this line
PUBLISH_COMMAND = '{0} setup.py sdist bdist_wheel upload -r pypi'.format(
    sys.executable)
GS_COMMAND = ('gs pyexcel-io v0.5.13 ' +
              "Find 0.5.13 in changelog for more details")
NO_GS_MESSAGE = ('Automatic github release is disabled. ' +
                 'Please install gease to enable it.')
UPLOAD_FAILED_MSG = (
    'Upload failed. please run "%s" yourself.' % PUBLISH_COMMAND)
HERE = os.path.abspath(os.path.dirname(__file__))


class PublishCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package on github and pypi'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds...')
            rmtree(os.path.join(HERE, 'dist'))
            rmtree(os.path.join(HERE, 'build'))
            rmtree(os.path.join(HERE, 'pyexcel_io.egg-info'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution...')
        run_status = True
        if has_gease():
            run_status = os.system(GS_COMMAND) == 0
        else:
            self.status(NO_GS_MESSAGE)
        if run_status:
            if os.system(PUBLISH_COMMAND) != 0:
                self.status(UPLOAD_FAILED_MSG % PUBLISH_COMMAND)

        sys.exit()


SETUP_COMMANDS.update({
    'publish': PublishCommand
})


def has_gease():
    """
    test if github release command is installed

    visit http://github.com/moremoban/gease for more info
    """
    try:
        import gease  # noqa
        return True
    except ImportError:
        return False


def read_files(*files):
    """Read files into setup"""
    text = ""
    for single_file in files:
        content = read(single_file)
        text = text + content + "\n"
    return text


def read(afile):
    """Read a file into setup"""
    the_relative_file = os.path.join(HERE, afile)
    with codecs.open(the_relative_file, 'r', 'utf-8') as opened_file:
        content = filter_out_test_code(opened_file)
        content = "".join(list(content))
        return content


def filter_out_test_code(file_handle):
    found_test_code = False
    for line in file_handle.readlines():
        if line.startswith('.. testcode:'):
            found_test_code = True
            continue
        if found_test_code is True:
            if line.startswith('  '):
                continue
            else:
                empty_line = line.strip()
                if len(empty_line) == 0:
                    continue
                else:
                    found_test_code = False
                    yield line
        else:
            for keyword in ['|version|', '|today|']:
                if keyword in line:
                    break
            else:
                yield line


if __name__ == '__main__':
    setup(
        test_suite="tests",
        name=NAME,
        author=AUTHOR,
        version=VERSION,
        author_email=EMAIL,
        description=DESCRIPTION,
        url=URL,
        download_url=DOWNLOAD_URL,
        long_description=read_files(*FILES),
        license=LICENSE,
        keywords=KEYWORDS,
        extras_require=EXTRAS_REQUIRE,
        tests_require=['nose'],
        install_requires=INSTALL_REQUIRES,
        packages=PACKAGES,
        include_package_data=True,
        zip_safe=False,
        classifiers=CLASSIFIERS,
        cmdclass=SETUP_COMMANDS
    )



"""
    pyexcel_io.fileformat.csvz
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    The lower level csvz file format handler.

    :copyright: (c) 2014-2017 by Onni Software Ltd.
    :license: New BSD License, see LICENSE for more details
"""
import zipfile

from pyexcel_io._compact import StringIO, PY2
from pyexcel_io.book import BookReader
from pyexcel_io.constants import FILE_FORMAT_CSVZ

from .csvr import CSVinMemoryReader, NamedContent


class CSVZipBookReader(BookReader):
    """csvz reader

    Read zipped csv file that was zipped up by pyexcel-io. It support
    single csv file and multiple csv files.
    """

    def __init__(self):
        BookReader.__init__(self)
        self._file_type = FILE_FORMAT_CSVZ
        self.zipfile = None

    def open(self, file_name, **keywords):
        BookReader.open(self, file_name, **keywords)
        self._native_book = self._load_from_file_alike_object(self._file_name)

    def open_stream(self, file_stream, **keywords):
        BookReader.open_stream(self, file_stream, **keywords)
        self._native_book = self._load_from_file_alike_object(
            self._file_stream
        )

    def read_sheet(self, native_sheet):
        content = self.zipfile.read(native_sheet.payload)
        if PY2:
            sheet = StringIO(content)
        else:
            sheet = StringIO(content.decode("utf-8"))

        reader = CSVinMemoryReader(
            NamedContent(native_sheet.name, sheet), **self._keywords
        )
        return reader.to_array()

    def close(self):
        if self.zipfile:
            self.zipfile.close()

    def _load_from_file_alike_object(self, file_alike_object):
        try:
            self.zipfile = zipfile.ZipFile(file_alike_object, "r")
            sheets = [
                NamedContent(_get_sheet_name(name), name)
                for name in self.zipfile.namelist()
            ]
            return sheets

        except zipfile.BadZipfile:
            print("StringIO instance was passed by any chance?")
            raise


def _get_sheet_name(filename):
    len_of_a_dot = 1
    len_of_csv_word = 3
    name_len = len(filename) - len_of_a_dot - len_of_csv_word
    return filename[:name_len]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from nose import SkipTest
from nose.tools import eq_
from pyexcel_io import get_data, save_data
from pyexcel_io._compact import PY26
import pyexcel as p

IN_TRAVIS = "TRAVIS" in os.environ


def test_issue_8():
    test_file = "test_issue_8.csv"
    data = [[1, 2], [], [], [], [3, 4]]
    save_data(test_file, data)
    written_data = get_data(test_file, skip_empty_rows=False)
    eq_(data, written_data[test_file])
    os.unlink(test_file)


def test_issue_20():
    test_file = get_fixture("issue20.csv")
    data = get_data(test_file)
    expected = [[u"to", u"infinity", u"and", u"beyond"]]
    eq_(data["issue20.csv"], expected)


def test_issue_23():
    test_file = get_fixture("issue23.csv")
    data = get_data(test_file)
    expected = [
        [8204235414504252, u"inf"],
        [82042354145042521, u"-inf"],
        [820423541450425216, 0],
        [820423541450425247, 1],
        [8204235414504252490, 1.1],
    ]
    eq_(data["issue23.csv"], expected)


# def test_issue_28():
#     from pyexcel_io.plugins import readers
#     from pyexcel_io.exceptions import UpgradePlugin
#     expected = "Please upgrade the plugin '%s' according to "
#     expected += "plugin compactibility table."
#     try:
#         readers.load_me_later('pyexcel_test')
#     except UpgradePlugin as e:
#         eq_(str(e), expected % 'pyexcel_test')


def test_issue_33_34():
    if PY26:
        pass
    else:
        import mmap

        test_file = get_fixture("issue20.csv")
        with open(test_file, "r+b") as f:
            memory_mapped_file = mmap.mmap(
                f.fileno(), 0, access=mmap.ACCESS_READ
            )
            data = get_data(memory_mapped_file, file_type="csv")
            expected = [[u"to", u"infinity", u"and", u"beyond"]]
            eq_(data["csv"], expected)


def test_issue_30_utf8_BOM_header():
    content = [[u"人有悲歡離合", u"月有陰晴圓缺"]]
    test_file = "test-utf8-BOM.csv"
    save_data(test_file, content, encoding="utf-8-sig", lineterminator="\n")
    custom_encoded_content = get_data(test_file, encoding="utf-8-sig")
    assert custom_encoded_content[test_file] == content
    with open(test_file, "rb") as f:
        content = f.read()
        assert content[0:3] == b"\xef\xbb\xbf"
    os.unlink(test_file)


def test_issue_33_34_utf32_encoded_file():
    if PY26:
        pass
    else:
        check_mmap_encoding("utf-32")


def test_issue_33_34_utf32be_encoded_file():
    if PY26:
        pass
    else:
        check_mmap_encoding("utf-32-be")


def test_issue_33_34_utf32le_encoded_file():
    if PY26:
        pass
    else:
        check_mmap_encoding("utf-32-le")


def test_issue_33_34_utf16_encoded_file():
    if PY26:
        pass
    else:
        check_mmap_encoding("utf-16")


def test_issue_33_34_utf16be_encoded_file():
    if PY26:
        pass
    else:
        check_mmap_encoding("utf-16-be")


def test_issue_33_34_utf16le_encoded_file():
    if PY26:
        pass
    else:
        check_mmap_encoding("utf-16-le")


def test_issue_33_34_utf8_encoded_file():
    if PY26:
        pass
    else:
        check_mmap_encoding("utf-8")


def check_mmap_encoding(encoding):
    import mmap

    content = [
        [u"Äkkilähdöt", u"Matkakirjoituksia", u"Matkatoimistot"],
        [u"Äkkilähdöt", u"Matkakirjoituksia", u"Matkatoimistot"],
    ]
    test_file = "test-%s-encoding-in-mmap-file.csv" % encoding
    save_data(test_file, content, encoding=encoding)
    with open(test_file, "r+b") as f:
        memory_mapped_file = mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ)
        data = get_data(memory_mapped_file, file_type="csv", encoding=encoding)
        eq_(data["csv"], content)

    os.unlink(test_file)


def test_issue_35_encoding_for_file_content():
    encoding = "utf-16"
    content = [
        [u"Äkkilähdöt", u"Matkakirjoituksia", u"Matkatoimistot"],
        [u"Äkkilähdöt", u"Matkakirjoituksia", u"Matkatoimistot"],
    ]
    test_file = "test-%s-encoding-in-mmap-file.csv" % encoding
    save_data(test_file, content, encoding=encoding)
    with open(test_file, "r+b") as f:
        csv_content = f.read()
        data = get_data(csv_content, file_type="csv", encoding=encoding)
        eq_(data["csv"], content)

    os.unlink(test_file)


def test_issue_43():
    # if not IN_TRAVIS:
    #    raise SkipTest()
    p.get_book(
        url="https://github.com/pyexcel/pyexcel-xls/raw/master/tests/fixtures/file_with_an_empty_sheet.xls"
    )
    # flake8: noqa


def test_pyexcel_issue_138():
    array = [["123_122", "123_1.", "123_1.0"]]
    save_data("test.csv", array)
    data = get_data("test.csv")
    expected = [["123_122", "123_1.", "123_1.0"]]
    eq_(data["test.csv"], expected)
    os.unlink("test.csv")


def get_fixture(file_name):
    return os.path.join("tests", "fixtures", file_name)



"""
    pyexcel_io.utils
    ~~~~~~~~~~~~~~~~~~~

    utility functions

    :copyright: (c) 2014-2017 by Onni Software Ltd.
    :license: New BSD License, see LICENSE for more details
"""
import pyexcel_io.constants as constants


XLS_PLUGIN = "pyexcel-xls"
XLSX_PLUGIN = "pyexcel-xlsx"
ODS_PLUGIN = "pyexcel-ods"
ODS3_PLUGIN = "pyexcel-ods3"
XLSXW_PLUGIN = "pyexcel-xlsxw"
IO_ITSELF = "pyexcel-io"


AVAILABLE_READERS = {
    constants.FILE_FORMAT_XLS: [XLS_PLUGIN],
    constants.FILE_FORMAT_XLSX: [XLS_PLUGIN, XLSX_PLUGIN],
    constants.FILE_FORMAT_XLSM: [XLS_PLUGIN, XLSX_PLUGIN],
    constants.FILE_FORMAT_ODS: [ODS_PLUGIN, ODS3_PLUGIN],
    constants.FILE_FORMAT_CSV: [IO_ITSELF],
    constants.FILE_FORMAT_TSV: [IO_ITSELF],
    constants.FILE_FORMAT_CSVZ: [IO_ITSELF],
    constants.FILE_FORMAT_TSVZ: [IO_ITSELF],
}

AVAILABLE_WRITERS = {
    constants.FILE_FORMAT_XLS: [XLS_PLUGIN],
    constants.FILE_FORMAT_XLSX: [XLSX_PLUGIN, XLSXW_PLUGIN],
    constants.FILE_FORMAT_XLSM: [XLSX_PLUGIN],
    constants.FILE_FORMAT_ODS: [ODS_PLUGIN, ODS3_PLUGIN],
    constants.FILE_FORMAT_CSV: [IO_ITSELF],
    constants.FILE_FORMAT_TSV: [IO_ITSELF],
    constants.FILE_FORMAT_CSVZ: [IO_ITSELF],
    constants.FILE_FORMAT_TSVZ: [IO_ITSELF],
}


def _index_filter(current_index, start, limit=-1):
    out_range = constants.SKIP_DATA
    if current_index >= start:
        out_range = constants.TAKE_DATA
    if limit > 0 and out_range == constants.TAKE_DATA:
        if current_index >= (start + limit):
            out_range = constants.STOP_ITERATION
    return out_range


def is_empty_array(array):
    """
    Check if an array is an array of '' or not
    """
    empty_array = [element for element in array if element != ""]
    return len(empty_array) == 0


def swap_empty_string_for_none(array):
    """ replace empty string fields with None """

    def swap(value):
        """ change empty string to None """
        if value == "":
            return None

        else:
            return value

    return [swap(x) for x in array]
